/*
* chsums/6q1g.cpp
*
* This file is part of NJet library
* Copyright (C) 2011, 2012 NJet Collaboration
*
* This software is distributed under the terms of the GNU General Public License (GPL)
*/

#include <cassert>

#include "6q1g.h"
#include "../ngluon2/Model.h"

// class Amp6q1g

template <typename T>
Amp6q1g<T>::Amp6q1g(const T scalefactor, const int mFC, const NJetAmpTables& tables)
  : BaseClass(mFC, tables)
{
  for (int fv=0; fv<mFC; fv++) {
    BaseClass::initNG(scalefactor, fv);
    BaseClass::setProcess(StandardModel::NGluon1compat(NN, getFlav(fv)), fv);
  }

  initNc();
}

template <typename T>
void Amp6q1g<T>::setNc(const ST Nc_)
{
  BaseClass::setNc(Nc_);
  initNc();
}

template <typename T>
void Amp6q1g<T>::initNc()
{
  Nmat[0] = 0.;
  Nmat[1] = 1.;
  Nmat[2] = Nc;
  Nmat[3] = Nc2;
  assert(3 < BaseClass::NmatLen);

  Nmatcc[0] = 0.;
  Nmatcc[1] = -V;
  Nmatcc[2] = 1.;
  Nmatcc[3] = -Nc2;
  Nmatcc[4] = -Nc;
  Nmatcc[5] = -Nc*V;
  Nmatcc[6] = -Nc3;
  Nmatcc[7] = Nc2;
  Nmatcc[8] = Nc;
  Nmatcc[9] = -Nc2*V;
  Nmatcc[10] = -Nc4;
  Nmatcc[11] = Nc3;
  assert(11 < BaseClass::NmatccLen);

  bornFactor = 0.125*V*V;
  loopFactor = 2.*bornFactor;
  bornccFactor = -0.5*V/Nc;
}

template <typename T>
void Amp6q1g<T>::getfvpart1_full(const int fv, LoopValue* fvpart)
{
  return getfvpart1_full_(fv, fvpart);
}

template <typename T>
void Amp6q1g<T>::getfvpart1_full(const int fv, LoopResult<T>* fvpart)
{
  return getfvpart1_full_(fv, fvpart);
}

// -------- BEGIN -- automatically generated code -- BEGIN -------- //

template <typename T>
void Amp6q1g<T>::getfvpart0(const int fv, TreeValue* fvpart, TreeValue* fvpartC)
{
  fvSet(fv);
  // 19 primitives
  const TreeValue P1234567 = A0(0,1,2,3,4,5,6);
  const TreeValue P1234576 = A0(0,1,2,3,4,6,5);
  const TreeValue P1234675 = A0(0,1,2,3,5,6,4);
  const TreeValue P1234756 = A0(0,1,2,3,6,4,5);
  const TreeValue P1237456 = A0(0,1,2,6,3,4,5);
  const TreeValue P1243765 = A0(0,1,3,2,6,5,4);
  const TreeValue P1256347 = A0(0,1,4,5,2,3,6);
  const TreeValue P1256734 = A0(0,1,4,5,6,2,3);
  const TreeValue P1257634 = A0(0,1,4,6,5,2,3);
  const TreeValue P1257643 = A0(0,1,4,6,5,3,2);
  const TreeValue P1265347 = A0(0,1,5,4,2,3,6);
  const TreeValue P1265743 = A0(0,1,5,4,6,3,2);
  const TreeValue P1273456 = A0(0,1,6,2,3,4,5);
  const TreeValue P1273465 = A0(0,1,6,2,3,5,4);
  const TreeValue P1275634 = A0(0,1,6,4,5,2,3);
  const TreeValue P1275643 = A0(0,1,6,4,5,3,2);
  const TreeValue P1724356 = A0(0,6,1,3,2,4,5);
  const TreeValue P1724365 = A0(0,6,1,3,2,5,4);
  const TreeValue P1725634 = A0(0,6,1,4,5,2,3);
  fvpart[0] = P1234567;
  fvpart[1] = P1234756;
  fvpart[2] = P1256347;
  fvpart[3] = P1256734;
  fvpart[4] = P1273456;
  fvpart[5] = P1275634;
  fvpart[6] = (-P1234756-P1237456-P1273456+P1275643)/Nc;
  fvpart[7] = (-P1256347-P1265743-P1275634-P1725634)/Nc;
  fvpart[8] = (P1234576+P1234756+P1237456-P1257643+P1273456-P1275643-P1724356+P1725634)/Nc;
  fvpart[9] = (P1234576+P1234756+P1237456-P1243765)/Nc;
  fvpart[10] = (-P1256734-P1257634+P1273465-P1275634)/Nc;
  fvpart[11] = (-P1234567-P1234576-P1234756+P1265347)/Nc;
  fvpart[12] = (-P1234675-P1234756+P1243765-P1256347-P1256734+P1257643-P1265347+P1265743-P1273456
    -P1273465+P1724356+P1724365)/Nc;
  fvpart[13] = (P1234675+P1257634)/Nc;
  fvpart[14] = (-P1234567-P1234576-P1234756-P1237456-P1273456-P1724365)/Nc;
  fvpart[15] = (-P1234576-P1257634)/Nc2;
  fvpart[16] = (-P1237456+P1256347+P1256734+P1257634+P1275634+P1725634)/Nc2;
  fvpart[17] = (P1234567+P1234576+P1234756+P1237456+P1273456-P1725634)/Nc2;

  if (fvpartC) {
    for (int i=0; i<C0; i++) {
      fvpartC[i] = conj(fvpart[i]);
    }
  }
}

template <typename T>
template <typename LT>
inline
void Amp6q1g<T>::getfvpart1_full_(const int fv, LT* fvpart)
{
  fvSet(fv);
  {
    // 192 primitives
    const LT P1234567 = AL(0,1,2,3,4,5,6);
    const LT P1234576 = AL(0,1,2,3,4,6,5);
    const LT P1234657 = AL(0,1,2,3,5,4,6);
    const LT P1234675 = AL(0,1,2,3,5,6,4);
    const LT P1234756 = AL(0,1,2,3,6,4,5);
    const LT P1234765 = AL(0,1,2,3,6,5,4);
    const LT P1235647 = AL(0,1,2,4,5,3,6);
    const LT P1235674 = AL(0,1,2,4,5,6,3);
    const LT P1235764 = AL(0,1,2,4,6,5,3);
    const LT P1237456 = AL(0,1,2,6,3,4,5);
    const LT P1237465 = AL(0,1,2,6,3,5,4);
    const LT P1237564 = AL(0,1,2,6,4,5,3);
    const LT P1243567 = AL(0,1,3,2,4,5,6);
    const LT P1243576 = AL(0,1,3,2,4,6,5);
    const LT P1243657 = AL(0,1,3,2,5,4,6);
    const LT P1243675 = AL(0,1,3,2,5,6,4);
    const LT P1243756 = AL(0,1,3,2,6,4,5);
    const LT P1243765 = AL(0,1,3,2,6,5,4);
    const LT P1245637 = AL(0,1,3,4,5,2,6);
    const LT P1245673 = AL(0,1,3,4,5,6,2);
    const LT P1245763 = AL(0,1,3,4,6,5,2);
    const LT P1247356 = AL(0,1,3,6,2,4,5);
    const LT P1247365 = AL(0,1,3,6,2,5,4);
    const LT P1247563 = AL(0,1,3,6,4,5,2);
    const LT P1253467 = AL(0,1,4,2,3,5,6);
    const LT P1253476 = AL(0,1,4,2,3,6,5);
    const LT P1253746 = AL(0,1,4,2,6,3,5);
    const LT P1256347 = AL(0,1,4,5,2,3,6);
    const LT P1256374 = AL(0,1,4,5,2,6,3);
    const LT P1256437 = AL(0,1,4,5,3,2,6);
    const LT P1256473 = AL(0,1,4,5,3,6,2);
    const LT P1256734 = AL(0,1,4,5,6,2,3);
    const LT P1256743 = AL(0,1,4,5,6,3,2);
    const LT P1257346 = AL(0,1,4,6,2,3,5);
    const LT P1257634 = AL(0,1,4,6,5,2,3);
    const LT P1257643 = AL(0,1,4,6,5,3,2);
    const LT P1263457 = AL(0,1,5,2,3,4,6);
    const LT P1263475 = AL(0,1,5,2,3,6,4);
    const LT P1263745 = AL(0,1,5,2,6,3,4);
    const LT P1265347 = AL(0,1,5,4,2,3,6);
    const LT P1265374 = AL(0,1,5,4,2,6,3);
    const LT P1265437 = AL(0,1,5,4,3,2,6);
    const LT P1265473 = AL(0,1,5,4,3,6,2);
    const LT P1265734 = AL(0,1,5,4,6,2,3);
    const LT P1265743 = AL(0,1,5,4,6,3,2);
    const LT P1267345 = AL(0,1,5,6,2,3,4);
    const LT P1267534 = AL(0,1,5,6,4,2,3);
    const LT P1267543 = AL(0,1,5,6,4,3,2);
    const LT P1273456 = AL(0,1,6,2,3,4,5);
    const LT P1273465 = AL(0,1,6,2,3,5,4);
    const LT P1273564 = AL(0,1,6,2,4,5,3);
    const LT P1274356 = AL(0,1,6,3,2,4,5);
    const LT P1274365 = AL(0,1,6,3,2,5,4);
    const LT P1274563 = AL(0,1,6,3,4,5,2);
    const LT P1275346 = AL(0,1,6,4,2,3,5);
    const LT P1275634 = AL(0,1,6,4,5,2,3);
    const LT P1275643 = AL(0,1,6,4,5,3,2);
    const LT P1276345 = AL(0,1,6,5,2,3,4);
    const LT P1276534 = AL(0,1,6,5,4,2,3);
    const LT P1276543 = AL(0,1,6,5,4,3,2);
    const LT P1342567 = AL(0,2,3,1,4,5,6);
    const LT P1342576 = AL(0,2,3,1,4,6,5);
    const LT P1342657 = AL(0,2,3,1,5,4,6);
    const LT P1342675 = AL(0,2,3,1,5,6,4);
    const LT P1342756 = AL(0,2,3,1,6,4,5);
    const LT P1342765 = AL(0,2,3,1,6,5,4);
    const LT P1345627 = AL(0,2,3,4,5,1,6);
    const LT P1345672 = AL(0,2,3,4,5,6,1);
    const LT P1345762 = AL(0,2,3,4,6,5,1);
    const LT P1346527 = AL(0,2,3,5,4,1,6);
    const LT P1346572 = AL(0,2,3,5,4,6,1);
    const LT P1346752 = AL(0,2,3,5,6,4,1);
    const LT P1347256 = AL(0,2,3,6,1,4,5);
    const LT P1347265 = AL(0,2,3,6,1,5,4);
    const LT P1347562 = AL(0,2,3,6,4,5,1);
    const LT P1347652 = AL(0,2,3,6,5,4,1);
    const LT P1356427 = AL(0,2,4,5,3,1,6);
    const LT P1356472 = AL(0,2,4,5,3,6,1);
    const LT P1356742 = AL(0,2,4,5,6,3,1);
    const LT P1357642 = AL(0,2,4,6,5,3,1);
    const LT P1374256 = AL(0,2,6,3,1,4,5);
    const LT P1374265 = AL(0,2,6,3,1,5,4);
    const LT P1374562 = AL(0,2,6,3,4,5,1);
    const LT P1374652 = AL(0,2,6,3,5,4,1);
    const LT P1375642 = AL(0,2,6,4,5,3,1);
    const LT P1562347 = AL(0,4,5,1,2,3,6);
    const LT P1562374 = AL(0,4,5,1,2,6,3);
    const LT P1562437 = AL(0,4,5,1,3,2,6);
    const LT P1562473 = AL(0,4,5,1,3,6,2);
    const LT P1562734 = AL(0,4,5,1,6,2,3);
    const LT P1562743 = AL(0,4,5,1,6,3,2);
    const LT P1563427 = AL(0,4,5,2,3,1,6);
    const LT P1563472 = AL(0,4,5,2,3,6,1);
    const LT P1563742 = AL(0,4,5,2,6,3,1);
    const LT P1567234 = AL(0,4,5,6,1,2,3);
    const LT P1567243 = AL(0,4,5,6,1,3,2);
    const LT P1567342 = AL(0,4,5,6,2,3,1);
    const LT P1576234 = AL(0,4,6,5,1,2,3);
    const LT P1576243 = AL(0,4,6,5,1,3,2);
    const LT P1576342 = AL(0,4,6,5,2,3,1);
    const LT P1723456 = AL(0,6,1,2,3,4,5);
    const LT P1723465 = AL(0,6,1,2,3,5,4);
    const LT P1723564 = AL(0,6,1,2,4,5,3);
    const LT P1724356 = AL(0,6,1,3,2,4,5);
    const LT P1724365 = AL(0,6,1,3,2,5,4);
    const LT P1724563 = AL(0,6,1,3,4,5,2);
    const LT P1725346 = AL(0,6,1,4,2,3,5);
    const LT P1725634 = AL(0,6,1,4,5,2,3);
    const LT P1725643 = AL(0,6,1,4,5,3,2);
    const LT P1726345 = AL(0,6,1,5,2,3,4);
    const LT P1726534 = AL(0,6,1,5,4,2,3);
    const LT P1726543 = AL(0,6,1,5,4,3,2);
    const LT P1734256 = AL(0,6,2,3,1,4,5);
    const LT P1734265 = AL(0,6,2,3,1,5,4);
    const LT P1734562 = AL(0,6,2,3,4,5,1);
    const LT P1734652 = AL(0,6,2,3,5,4,1);
    const LT P1735642 = AL(0,6,2,4,5,3,1);
    const LT P1756234 = AL(0,6,4,5,1,2,3);
    const LT P1756243 = AL(0,6,4,5,1,3,2);
    const LT P1756342 = AL(0,6,4,5,2,3,1);
    const LT P3124567 = AL(2,0,1,3,4,5,6);
    const LT P3124576 = AL(2,0,1,3,4,6,5);
    const LT P3124657 = AL(2,0,1,3,5,4,6);
    const LT P3124675 = AL(2,0,1,3,5,6,4);
    const LT P3124756 = AL(2,0,1,3,6,4,5);
    const LT P3124765 = AL(2,0,1,3,6,5,4);
    const LT P3125647 = AL(2,0,1,4,5,3,6);
    const LT P3125674 = AL(2,0,1,4,5,6,3);
    const LT P3125764 = AL(2,0,1,4,6,5,3);
    const LT P3126547 = AL(2,0,1,5,4,3,6);
    const LT P3126574 = AL(2,0,1,5,4,6,3);
    const LT P3126754 = AL(2,0,1,5,6,4,3);
    const LT P3127456 = AL(2,0,1,6,3,4,5);
    const LT P3127465 = AL(2,0,1,6,3,5,4);
    const LT P3127564 = AL(2,0,1,6,4,5,3);
    const LT P3127654 = AL(2,0,1,6,5,4,3);
    const LT P3156247 = AL(2,0,4,5,1,3,6);
    const LT P3156274 = AL(2,0,4,5,1,6,3);
    const LT P3156724 = AL(2,0,4,5,6,1,3);
    const LT P3157624 = AL(2,0,4,6,5,1,3);
    const LT P3172456 = AL(2,0,6,1,3,4,5);
    const LT P3172465 = AL(2,0,6,1,3,5,4);
    const LT P3172564 = AL(2,0,6,1,4,5,3);
    const LT P3172654 = AL(2,0,6,1,5,4,3);
    const LT P3175624 = AL(2,0,6,4,5,1,3);
    const LT P3451267 = AL(2,3,4,0,1,5,6);
    const LT P3451276 = AL(2,3,4,0,1,6,5);
    const LT P3451726 = AL(2,3,4,0,6,1,5);
    const LT P3457126 = AL(2,3,4,6,0,1,5);
    const LT P3461257 = AL(2,3,5,0,1,4,6);
    const LT P3461275 = AL(2,3,5,0,1,6,4);
    const LT P3461725 = AL(2,3,5,0,6,1,4);
    const LT P3467125 = AL(2,3,5,6,0,1,4);
    const LT P3475126 = AL(2,3,6,4,0,1,5);
    const LT P3476125 = AL(2,3,6,5,0,1,4);
    const LT P3561247 = AL(2,4,5,0,1,3,6);
    const LT P3561274 = AL(2,4,5,0,1,6,3);
    const LT P3561724 = AL(2,4,5,0,6,1,3);
    const LT P3567124 = AL(2,4,5,6,0,1,3);
    const LT P3576124 = AL(2,4,6,5,0,1,3);
    const LT P3712456 = AL(2,6,0,1,3,4,5);
    const LT P3712465 = AL(2,6,0,1,3,5,4);
    const LT P3712564 = AL(2,6,0,1,4,5,3);
    const LT P3712654 = AL(2,6,0,1,5,4,3);
    const LT P3715624 = AL(2,6,0,4,5,1,3);
    const LT P3745126 = AL(2,6,3,4,0,1,5);
    const LT P3746125 = AL(2,6,3,5,0,1,4);
    const LT P3756124 = AL(2,6,4,5,0,1,3);
    const LT P5123467 = AL(4,0,1,2,3,5,6);
    const LT P5123476 = AL(4,0,1,2,3,6,5);
    const LT P5123746 = AL(4,0,1,2,6,3,5);
    const LT P5124367 = AL(4,0,1,3,2,5,6);
    const LT P5124376 = AL(4,0,1,3,2,6,5);
    const LT P5124736 = AL(4,0,1,3,6,2,5);
    const LT P5127346 = AL(4,0,1,6,2,3,5);
    const LT P5127436 = AL(4,0,1,6,3,2,5);
    const LT P5134267 = AL(4,0,2,3,1,5,6);
    const LT P5134276 = AL(4,0,2,3,1,6,5);
    const LT P5134726 = AL(4,0,2,3,6,1,5);
    const LT P5137426 = AL(4,0,2,6,3,1,5);
    const LT P5172346 = AL(4,0,6,1,2,3,5);
    const LT P5172436 = AL(4,0,6,1,3,2,5);
    const LT P5173426 = AL(4,0,6,2,3,1,5);
    const LT P5341267 = AL(4,2,3,0,1,5,6);
    const LT P5341276 = AL(4,2,3,0,1,6,5);
    const LT P5341726 = AL(4,2,3,0,6,1,5);
    const LT P5347126 = AL(4,2,3,6,0,1,5);
    const LT P5374126 = AL(4,2,6,3,0,1,5);
    const LT P5712346 = AL(4,6,0,1,2,3,5);
    const LT P5712436 = AL(4,6,0,1,3,2,5);
    const LT P5713426 = AL(4,6,0,2,3,1,5);
    const LT P5734126 = AL(4,6,2,3,0,1,5);

    fvpart[0] = Nc*P1234567+(-2.*P1234567+P1234576-P1234657-P1234675+P1234756+P1234765-P1235764+P1237456
    +P1237465-P1237564-2.*P1243567-P1243657-P1243675-P1245637-P1245673+P1253467-P1256437
    -P1256473-P1256743-P1265437-P1265473-P1265743-P1267543+P1273456+P1273465-P1273564
    +P1276345+P1276534+P1276543+P1342567+P1342657+P1342675-P1345762-P1347562-P1347652
    +P1357642-P1374562-P1374652+P1375642-P1576234-P1576243+P1576342+P1723456+P1723465
    -P1723564+P1726345+P1726534+P1726543-P1734562-P1734652+P1735642-P1756234-P1756243
    +P1756342-P3124576-P3124756-P3124765-P3127456-P3127465-P3127654+P3157624-P3172456
    -P3172465-P3172654+P3175624-P3451276-P3451726-P3467125-P5123476-P5123746-P5124376
    -P5124736-P5127346-P5127436+P5134276+P5134726+P5137426-P5172346-P5172436+P5173426
    +P5341276+P5341726)/Nc;
    fvpart[1] = Nc*P1234756+(P1234567+P1234576+P1234657-P1234675-2.*P1234756-P1234765-P1235674-P1235764
    +P1237456+P1237465+P1243567+P1243576+P1243657-P1243756-P1247356-P1247563-P1253746
    +P1256743-P1257346+P1257643-P1263475+P1265374+P1265734+2.*P1265743+P1273456+P1273465
    +P1274356+P1274365-P1342567-P1342576-P1342657+P1346752+P1347562+P1347652+P1356742
    +P1357642-P1374256-P1374265-P1562374-P1562734-P1562743+P1563742-P1567234-P1567243
    +P1567342-P1576234-P1576243+P1576342+P1723456+P1723465+P1724356+P1724365-P1734256
    -P1734265+P3124675+P3124756+P3124765-P3126574+P3156274+P3156724+P3157624+P3461257
    +P3475126+P3746125-P5123746-P5127346-P5127436+P5137426-P5172346-P5172436+P5173426
    +P5374126-P5712346-P5712436+P5713426+P5734126)/Nc;
    fvpart[2] = Nc*P1256347+(-P1234657-P1234675-P1234765+P1235647-P1243657-P1243675-P1243765-P1247365
    -P1253746-2.*P1256347+P1256374-P1256437-P1256473+P1256734+P1256743-P1257346+P1257634
    +P1257643-P1263457-P1263475-2.*P1265347-P1265437-P1265473+P1274356+P1274365+P1274563
    -P1275346+P1275634+P1275643+P1356742+P1357642-P1374256-P1374265-P1374652+P1375642
    +P1562347+P1562437+P1562473+P1724356+P1724365+P1724563-P1725346+P1725634+P1725643
    -P1734256-P1734265-P1734652+P1735642-P3125674-P3125764-P3126574-P3126754-P3127456
    -P3127465-P3127564-P3127654+P3156274+P3156724+P3157624-P3172456-P3172465-P3172564
    -P3172654+P3175624-P3451267-P3451276-P3451726-P3467125-P3476125+P3561274+P3561724
    -P3745126-P5127436+P5137426-P5172436+P5173426)/Nc;
    fvpart[3] = Nc*P1256734+(P1234765-P1235764+P1237465-P1237564+P1243576+P1243756+2.*P1243765-P1245673
    -P1253476-P1253746+P1256347+P1256374+P1256437-P1256473-2.*P1256734-P1256743+P1257634
    +P1257643+P1265347+P1265374+P1265437-P1265734-P1267345-P1267534+P1275634+P1275643
    +P1276534+P1276543-P1342576-P1342756-P1342765-P1347256-P1347265-P1347652+P1357642
    -P1374256-P1374265-P1374652+P1375642-P1562347-P1562374-P1562437-P1576234-P1576243
    +P1725634+P1725643+P1726534+P1726543-P1756234-P1756243+P3124567+P3124657+P3124675
    -P3125764-P3127564-P3127654+P3157624-P3172564-P3172654+P3175624+P3451267+P3476125
    +P3576124-P3712564-P3712654+P3715624+P3746125+P3756124-P5124376+P5134276+P5134726
    +P5137426)/Nc;
    fvpart[4] = Nc*P1273456+(P1234567+P1234576+P1234756+P1237456+P1243567+P1243576+P1243756+P1245637
    -P1247356-P1247365-P1253467-P1253476-P1253746+P1256437+P1265437-P1265473-P1265734
    -P1265743-P1267345-P1267534-P1267543-2.*P1273456-2.*P1273465+P1273564-P1274356-P1274365
    -P1276345-P1276534-P1276543-P1345672-P1345762-P1346572-P1346752-P1347256-P1347265
    -P1347562-P1347652+P1356472+P1356742+P1357642-P1374256-P1374265-P1374562-P1374652
    +P1375642+P1562473+P1562734+P1562743+P1563472+P1563742+P1723456+P1724356+P1724563
    -P1725346+P1725643+P1726543+P3156724+P3157624-P3172456-P3172465-P3172654+P3175624
    -P3451726-P3457126-P3461257-P3461275-P3475126-P3712456-P3712465-P3712654+P3715624
    -P3745126+P5124736+P5127346+P5127436+P5134726+P5137426+P5341726+P5347126+P5374126)/Nc;
    fvpart[5] = Nc*P1275634+(P1234657-P1235647-P1235674-P1235764+P1243657-P1243675-P1243756-P1243765
    -P1247356-P1247365-P1247563+P1256347+P1256374+P1256734+P1257634+P1263457+P1265347
    +P1265374+P1265734-P1267534-P1267543-P1274356-P1274365-P1274563+P1275346-2.*P1275634
    -2.*P1275643-P1276534-P1276543+P1342675+P1342756+P1342765-P1346572+P1356472+P1356742
    +P1357642-P1567234-P1567243-P1576234-P1576243+P1723465-P1723564+P1724365+P1725634
    +P1726345+P1726534-P3124567-P3124576-P3124657+P3126754+P3127564+P3127654+P3156724
    +P3157624-P3172456-P3172465-P3451726-P3457126-P3461275+P3561724+P3567124+P3576124
    -P3712456-P3712465+P5134726+P5137426-P5172436+P5173426-P5712436+P5713426)/Nc;
    fvpart[6] = P1243756+P1247356+P1247563+P1274356+P1274563+P1275643+P3124657+P3126547+P3126574
    -P3156247-P3156274-2.*P3156724-2.*P3157624+P3172465+P3172654-P3175624-P3561724-P3567124
    -P3576124+P3712465+P3712654-P3715624+(-P1234567-P1234576-P1234657-P1243567-P1243576
    -P1243657+P1253467+P1253476+P1253746+P1257346-P1263457+P1263475+P1263745+P1267345
    +P1267534+P1267543-P1275346+P1275634+P1275643+P1276345+P1276534+P1276543+P1342567
    +P1342576+P1342657-P1342675-P1342756-P1342765+P1345672+P1345762+2.*P1346572+P1347256
    +P1347265-P1356472-P1356742-P1357642+P1374256+P1374265-P1563472-P1563742+P1567234
    +P1567243-P1567342+P1576234+P1576243-P1576342-P1723456-P1723465-P1724356-P1724365
    +P1725346-P1726345+P1734256+P1734265-P3126754-P3127564-P3127654-P3156724-P3157624
    +P3451726+P3457126+P3461275-2.*P5134726-2.*P5137426+P5172436-2.*P5173426-P5341726-P5347126
    -P5374126+P5712436-2.*P5713426-P5734126)/Nc2;
    fvpart[7] = -P1243657-P1265743-P1274365+P1342657+P1346527+P1346572-P1356427-P1356472-2.*P1356742
    -2.*P1357642+P1374265+P1374652-P1375642+P1562743-P1563742+P1567243-P1567342+P1576243
    -P1576342-P1724365+P1734265+P1734652-P1735642+(P1234675+P1234756+P1234765+P1235674
    +P1235764+P1243675+P1243756+P1243765+P1247356+P1247365+P1247563+P1253746-P1256374
    -P1256734-P1256743+P1257346-P1257634-P1257643+P1263475-P1265374-P1265734-P1265743
    -P1346752-P1347562-P1347652-P1356742-P1357642+P3124567+P3124576+P3124657-P3124675
    -P3124756-P3124765+P3125674+P3125764+2.*P3126574+P3127456+P3127465-P3156274-P3156724
    -P3157624+P3172456+P3172465+P3451267+P3451276+P3451726+P3457126-P3461257+P3461275
    +P3461725+P3467125-P3475126+P3476125-P3561274-P3561724-P3567124-P3576124+P3712456
    +P3712465+P3745126-P3746125+P5123746+P5127346+2.*P5127436-P5137426+P5172346+2.*P5172436
    -P5173426-P5374126+P5712346+2.*P5712436-P5713426-P5734126)/Nc2;
    fvpart[8] = P1243567+P1245637+P1245673+P1256437+P1256473+P1256743+P3124765+P3125647+P3125674
    +P3125764+P3126547+P3126574+P3126754+P3127465+P3127564+2.*P3127654-P3156247-P3156274
    -P3156724-2.*P3157624+P3172465+P3172564+2.*P3172654-2.*P3175624-P3561247-P3561274-P3561724
    -P3576124+P3712564+P3712654-P3715624-P3756124+(P1234567+P1234657+P1234675+P1243567
    +P1243657+P1243675-P1253467+P1253476+P1253746+P1257346-P1257634-P1257643+P1263457
    +P1263475+P1263745+P1267345+P1275346-P1275634-P1275643-P1276345-P1276534-P1276543
    -P1342567+P1342576-P1342657-P1342675+P1342756+P1342765+P1345762+P1347256+P1347265
    +P1347562+2.*P1347652-P1357642+P1374256+P1374265+P1374562+2.*P1374652-P1375642+P1576234
    +P1576243-P1576342+P1725346-P1725634-P1725643-P1726345-P1726534-P1726543+P1734256
    +P1734265+P1734562+2.*P1734652-P1735642+P1756234+P1756243-P1756342+P3125764+P3127564
    +P3127654-P3157624+P3172564+P3172654-P3175624+P3451276+P3451726+P3467125+P5124376
    +P5124736+P5127436-2.*P5134276-2.*P5134726-2.*P5137426+P5172436-2.*P5173426-P5341276-P5341726)/Nc2;
    fvpart[9] = -P1243765-P1265437-P1276543+P1342765+P1345627+P1345672+P1345762+P1346527+P1346572
    +P1346752+P1347265+P1347562+2.*P1347652-P1356427-P1356472-P1356742-2.*P1357642+P1374265
    +P1374562+2.*P1374652-2.*P1375642+P1562437-P1563427-P1563472-P1563742+P1576243-P1576342
    -P1726543+P1734562+P1734652-P1735642+P1756243-P1756342+(-P1234576-P1234756-P1234765
    +P1235764-P1237456-P1237465+P1237564-P1243576-P1243756-P1243765+P1245673+P1253476
    +P1253746+P1256473+P1256734+P1256743+P1265473+P1265734+P1265743+P1267345+P1267534
    +P1267543+P1345762+P1347562+P1347652-P1357642+P1374562+P1374652-P1375642-P3124567
    +P3124576-P3124657-P3124675+P3124756+P3124765+P3125764+P3127456+P3127465+P3127564
    +2.*P3127654-P3157624+P3172456+P3172465+P3172564+2.*P3172654-P3175624-P3451267+P3451276
    +P3451726+P3457126+P3461257+P3461275+P3461725+P3467125+P3475126-P3476125-P3576124
    +P3712456+P3712465+P3712564+2.*P3712654-P3715624+P3745126-P3746125-P3756124+P5123476
    +P5123746+2.*P5124376-P5134276-P5134726-P5137426-P5341276-P5341726-P5347126-P5374126)/Nc2;
    fvpart[10] = P1265734+P1267345+P1267534+P1273465+P1276345+P1276534+P3461257+P3461275+P5124367
    +P5124376-P5134267-P5134276-2.*P5134726-2.*P5137426+P5172436-P5173426-P5341726-P5347126
    -P5374126+P5712436-P5713426+(P1235647+P1235674+P1235764+P1237564-P1245637+P1245673
    +P1245763+P1247356+P1247365+P1247563-P1256347-P1256374-P1256437-P1265347-P1265374
    -P1265437+P1273456+P1273465-P1273564+P1274356+P1274365+P1274563+P1346572+P1346752
    +P1347256+P1347265+P1347652-2.*P1356472-2.*P1356742-2.*P1357642+P1374256+P1374265+P1374652
    -2.*P1375642+P1562347+P1562374+P1562437-P1562473-P1562734-P1562743-P1563472-P1563742
    +P1567234+P1567243+P1576234+P1576243+P1723564-P1724563-P1725634-P1725643-P1726534
    -P1726543+P1756234+P1756243-2.*P3156724-2.*P3157624+P3172456+P3172465+P3172654-2.*P3175624
    -P3561724-P3567124-P3576124+P3712456+P3712465+P3712654-2.*P3715624-P3756124-P5124736
    -P5127346-P5127436-P5134726-P5137426)/Nc2;
    fvpart[11] = P1234657+P1234675+P1234765+P1263457+P1263475+P1265347+P3467125+P3476125+P5123467
    +P5123476+P5123746+P5124367+P5124376+P5124736+P5127346+2.*P5127436-P5134267-P5134276
    -P5134726-2.*P5137426+P5172346+2.*P5172436-2.*P5173426-P5341267-P5341276-P5341726-P5374126
    +P5712346+P5712436-P5713426-P5734126+(-P1235647+P1235674+P1235764-P1237456-P1237465
    +P1237564+P1245637+P1245673+P1245763+P1247563+P1256347+P1256437+P1256473+P1265347
    +P1265437+P1265473-P1273456-P1273465+P1273564-P1274356-P1274365-P1274563-2.*P1356742
    -2.*P1357642+P1374256+P1374265+P1374652-2.*P1375642-P1562347+P1562374-P1562437-P1562473
    +P1562734+P1562743-P1563742+P1567234+P1567243-P1567342+P1576234+P1576243-P1576342
    -P1723456-P1723465+P1723564-P1724356-P1724365-P1724563+P1734256+P1734265+P1734652
    -2.*P1735642+P1756234+P1756243-P1756342+P3126574+P3126754+P3127456+P3127465+P3127654
    -2.*P3156274-2.*P3156724-2.*P3157624+P3172456+P3172465+P3172654-2.*P3175624-P3561274-P3561724
    +P5123746+P5127346+P5127436-P5137426+P5172346+P5172436-P5173426)/Nc2;
    fvpart[12] = -P1243567-P1243576-P1243756-P1245637+P1253467+P1253476+P1253746-P1256437-P1256743
    +P1257346-P1257643-P1274356-P1274563+P1275346-P1275643+P1342567+P1342576+P1342756
    +P1347256+P1374256-P1724356-P1724563+P1725346-P1725643+P1734256-P3124657-P3124675
    -P3124765-P3126547+P3156247+P3561247+(-P1237456-P1237465+P1247356+P1247365+2.*P1253746
    -P1256374+P1256473-2.*P1263745-P1265374+P1265473-P1356742-P1357642+2.*P1374256+2.*P1374265
    +P1374562+2.*P1374652-P1375642+P1562374-P1562473-P1563742+P3125674+P3125764+P3126574
    +P3126754+P3127564+P3127654-P3156274-P3156724-P3157624+P3172564+P3172654-P3175624
    +P3712564+P3712654-P3715624+P3745126-P3746125-P5124736-2.*P5137426-P5374126)/Nc2;
    fvpart[13] = -P1234657-P1234765+P1235647+P1235674+P1235764-P1237465+P1237564-P1263457-P1265347
    -P1265374-P1265734-P1273465+P1273564-P1276345-P1276534+P1562347+P1562374+P1562734
    +P1567234+P1576234-P1723465+P1723564-P1726345-P1726534+P1756234-P3461257-P3476125
    -P3746125-P5124367+P5134267+P5341267+(-P1234576+P1234675+2.*P1235764-P1243576+P1243675
    -2.*P1245763-P1257634-P1257643+P1267534+P1267543+P1342576-P1342675-P1346752-2.*P1357642
    +2.*P1576234+2.*P1576243-P1576342+P3124576-P3124675-P3126754-2.*P3157624-P3576124+P5123476
    +P5123746+P5124376+P5124736+P5127346+P5127436-P5134276-P5134726-P5137426+P5172346
    +P5172436-P5173426+P5712346+P5712436-P5713426)/Nc2;
    fvpart[14] = P1243657+P1243675+P1243765+P1247365+P1265437+P1265473+P1265743+P1267543+P1274365
    +P1276543-P1342657-P1342675-P1342765-P1346527+P1356427-P1562437-P1562473-P1562743
    +P1563427+P3124567+P3124576+P3124756+P3127456+P3172456+P3451267+P3451276+P3451726
    +P3457126+P3475126+P3712456+P3745126+(P1345672+P1345762+P1346572+P1346752+P1347562
    +P1347652-P1356472-P1356742-P1357642+P1374562+P1374652-P1375642-P1723456-P1723465
    +P1723564-P1724356-P1724365-P1724563+P1725346-P1725634-P1725643-P1726345-P1726534
    -P1726543+P1734562+P1734652-P1735642-P3156724-P3157624+2.*P3172456+2.*P3172465+P3172564
    +2.*P3172654-P3175624+2.*P3451726-2.*P3461725-P3561724-P5134726-P5137426+P5172346+2.*P5172436
    -P5173426-P5341726)/Nc2;
    fvpart[15] = (P1245763+P1263457+P1276345-P1342657-P1342765-P1345627-P1345672-P1345762-2.*P1346527
    -2.*P1346572-P1346752-P1347265-P1347562-2.*P1347652+P1356427+P1356472+P1356742+2.*P1357642
    -P1374265-P1374562-2.*P1374652+P1375642+P1563427+P1563472+P1563742+P1567342-P1576243
    +2.*P1576342+P1726345-P1734265-P1734562-2.*P1734652+P1735642+P1756342-P3124657-P3124765
    -P3125647-P3125674-P3125764-2.*P3126547-2.*P3126574-P3126754-P3127465-P3127564-2.*P3127654
    +P3156247+P3156274+P3156724+2.*P3157624-P3172465-P3172564-2.*P3172654+P3175624+P3461257
    +P3476125+P3561247+P3561274+P3561724+P3567124+2.*P3576124-P3712465-P3712564-2.*P3712654
    +P3715624+P3746125+P3756124+P5124367-P5134267-P5341267)/Nc+(P1234576-P1234675-P1235764
    +P1243576-P1243675+P1245763-P1253476-P1253746-P1257346+P1257634+P1257643-P1263475
    -P1263745-P1267345-P1267534-P1267543-P1342576+P1342675-P1345762+2.*P1346752+2.*P1357642
    -P1576234-P1576243+P1576342-P3124576+P3124675-P3125764+2.*P3126754+2.*P3157624-P3451276
    -P3451726-P3457126-P3461275-P3461725-P3467125+P3576124-P5123476-P5123746-2.*P5124376
    -2.*P5124736-P5127346-2.*P5127436+2.*P5134276+2.*P5134726+2.*P5137426-P5172346-2.*P5172436
    +2.*P5173426+P5341276+P5341726+P5347126+P5374126-P5712346-2.*P5712436+2.*P5713426+P5734126)/Nc3;
    fvpart[16] = (P1245637+P1263745+P1274563-P1346527-P1346572-P1346752-P1347652+2.*P1356427+2.*P1356472
    +3.*P1356742+3.*P1357642-P1374265-2.*P1374652+3.*P1375642-P1562437-P1562743+P1563427+P1563472
    +2.*P1563742-P1567243+P1567342-P1576243+P1576342+P1724563-P1734652+2.*P1735642-P1756243
    +P1756342+P3124657+P3124675+P3124765+P3126547-P3156247-P3461257-P3461275-P3461725
    -P3467125-P3476125-P3561247-P5123467-P5123476-P5123746-2.*P5124367-2.*P5124376-P5124736
    -P5127346-2.*P5127436+P5134267+P5134276+P5134726+2.*P5137426-P5172346-2.*P5172436+P5173426
    +P5341267+P5341276+P5341726+P5347126+2.*P5374126-P5712346-2.*P5712436+P5713426+P5734126)/Nc
    +(-P1235674-P1235764+P1237456+P1237465-P1237564-P1245673-P1245763-P1247356-P1247365
    -P1247563-P1253746+P1256374-P1256473+P1263745+P1265374-P1265473+2.*P1356742+2.*P1357642
    -P1374256-P1374265-P1374562-2.*P1374652+2.*P1375642-P1562374+P1562473+P1563742-P3125674
    -P3125764-2.*P3126574-2.*P3126754-P3127456-P3127465-P3127564-2.*P3127654+2.*P3156274+2.*P3156724
    +2.*P3157624-P3172456-P3172465-P3172564-2.*P3172654+2.*P3175624+P3561274+P3561724+P3567124
    +P3576124-P3712456-P3712465-P3712564-2.*P3712654+2.*P3715624-P3745126+P3746125+P3756124
    -P5123746+2.*P5124736+2.*P5137426+P5374126)/Nc3;
    fvpart[17] = (-P1245637-P1245673-P1245763-P1247563-P1263457-P1263475-P1263745-P1267345-P1274563
    -P1276345+P1342657+P1342675+P1342765+P1346527-P1356427+P1562437+P1562473+P1562743
    -P1563427-P3126547-P3126574-P3126754-P3127654+2.*P3156247+2.*P3156274+3.*P3156724+3.*P3157624
    -P3172465-2.*P3172654+3.*P3175624+P3461725+P3561247+P3561274+2.*P3561724+P3567124+P3576124
    -P3712654+2.*P3715624+P3756124-P5124367-P5124376-P5124736-P5127436+2.*P5134267+2.*P5134276
    +3.*P5134726+3.*P5137426-2.*P5172436+3.*P5173426+P5341267+P5341276+2.*P5341726+P5347126
    +P5374126-P5712436+2.*P5713426+P5734126)/Nc+(-P1345672-P1345762-2.*P1346572-2.*P1346752
    -P1347256-P1347265-P1347562-2.*P1347652+2.*P1356472+2.*P1356742+2.*P1357642-P1374256-P1374265
    -P1374562-2.*P1374652+2.*P1375642+P1563472+P1563742-P1567234-P1567243+P1567342-P1576234
    -P1576243+P1576342+P1723456+P1723465-P1723564+P1724356+P1724365+P1724563-P1725346
    +P1725634+P1725643+P1726345+P1726534+P1726543-P1734256-P1734265-P1734562-2.*P1734652
    +2.*P1735642-P1756234-P1756243+P1756342+2.*P3156724+2.*P3157624-P3172456-P3172465-P3172564
    -2.*P3172654+2.*P3175624-P3451726+P3461725+P3561724+2.*P5134726+2.*P5137426-P5172346-2.*P5172436
    +2.*P5173426+P5341726)/Nc3;

  }
  if (Nf != 0.) {
    // 20 primitives
    const LT Q1234567 = AF(0,1,2,3,4,5,6);
    const LT Q1234576 = AF(0,1,2,3,4,6,5);
    const LT Q1234675 = AF(0,1,2,3,5,6,4);
    const LT Q1234756 = AF(0,1,2,3,6,4,5);
    const LT Q1237456 = AF(0,1,2,6,3,4,5);
    const LT Q1243765 = AF(0,1,3,2,6,5,4);
    const LT Q1256347 = AF(0,1,4,5,2,3,6);
    const LT Q1256734 = AF(0,1,4,5,6,2,3);
    const LT Q1257634 = AF(0,1,4,6,5,2,3);
    const LT Q1257643 = AF(0,1,4,6,5,3,2);
    const LT Q1265347 = AF(0,1,5,4,2,3,6);
    const LT Q1265743 = AF(0,1,5,4,6,3,2);
    const LT Q1273456 = AF(0,1,6,2,3,4,5);
    const LT Q1273465 = AF(0,1,6,2,3,5,4);
    const LT Q1275634 = AF(0,1,6,4,5,2,3);
    const LT Q1275643 = AF(0,1,6,4,5,3,2);
    const LT Q1724356 = AF(0,6,1,3,2,4,5);
    const LT Q1724365 = AF(0,6,1,3,2,5,4);
    const LT Q1725634 = AF(0,6,1,4,5,2,3);
    const LT Q1726543 = AF(0,6,1,5,4,3,2);

    fvpart[0] += Nf*(-Q1234567);
    fvpart[1] += Nf*(-Q1234756);
    fvpart[2] += Nf*(-Q1256347);
    fvpart[3] += Nf*(-Q1256734);
    fvpart[4] += Nf*(-Q1273456);
    fvpart[5] += Nf*(-Q1275634);
    fvpart[6] += Nf*((Q1234756+Q1237456+Q1273456-Q1275643)/Nc);
    fvpart[7] += Nf*((Q1256347+Q1265743+Q1275634+Q1725634)/Nc);
    fvpart[8] += Nf*((2.*Q1234567+Q1234576+Q1234756+Q1237456+Q1257643+Q1273456+Q1275643+Q1724356+2.*Q1724365
    +Q1725634-2.*Q1726543)/Nc);
    fvpart[9] += Nf*((Q1234567+Q1243765+Q1273456+Q1724365+Q1725634-Q1726543)/Nc);
    fvpart[10] += Nf*((Q1256734+Q1257634-Q1273465+Q1275634)/Nc);
    fvpart[11] += Nf*((Q1234567+Q1234576+Q1234756-Q1265347)/Nc);
    fvpart[12] += Nf*((Q1234675+Q1234756-Q1243765+Q1256347+Q1256734-Q1257643+Q1265347-Q1265743+Q1273456
    +Q1273465-Q1724356-Q1724365)/Nc);
    fvpart[13] += Nf*((-Q1234675-Q1257634)/Nc);
    fvpart[14] += Nf*((-Q1725634+Q1726543)/Nc);
    fvpart[15] += Nf*(-(2.*Q1234567+Q1234576+2.*Q1234756+2.*Q1237456-Q1257634+2.*Q1273456+2.*Q1724365+2.*Q1725634
    -2.*Q1726543)/Nc2);
    fvpart[16] += Nf*(-(Q1234567+Q1234576+Q1234756+Q1256347+Q1256734+Q1257634+Q1273456+Q1275634+Q1724365
    +2.*Q1725634-Q1726543)/Nc2);
    fvpart[17] += Nf*(-(2.*Q1234567+2.*Q1234576+2.*Q1234756+2.*Q1237456+2.*Q1273456+Q1724365-Q1726543)/Nc2);

  }
}

// --------- END --- automatically generated code --- END --------- //

#ifdef USE_SD
  template class Amp6q1g<double>;
#endif
#ifdef USE_DD
  template class Amp6q1g<dd_real>;
#endif
#ifdef USE_QD
  template class Amp6q1g<qd_real>;
#endif
#ifdef USE_VC
  template class Amp6q1g<Vc::double_v>;
#endif
